classdef EPS1000

   properties
   
      model = "EPS1000"
      version = "0.9.0"
      
   end
   
   methods(Static)
   
      %%%%%%%%%%%%%%%%%%%%%%%
      % Basic communication %
      %%%%%%%%%%%%%%%%%%%%%%%

      function init(ARG)
         % EPS1000.init('192.168.1.100');    opens LAN device
         % EPS1000.init(1);                  opens last USB device
         % EPS1000.init(0);                  shows list of USB devices and lets user chose one
         % EPS1000.init;                     same as eps=EPS1000(1);
         % PM1000.init;EPS1000('PMS');       connects to eps card inside a PMS1000;

         if (nargin<1)
            ARG=1; % connect to last device
         end;
         
         global EPSHandle
         if length(ARG)==3&&sum(ARG=='PMS')==3  % via PMS1000 device
            global PMHandle
            EPSHandle = PMHandle
         elseif length(ARG)>6&&sum(class(ARG)=='char')==4  % LAN DEVICE
            EPSHandle=NOVTCP(ARG);
         else % USB DEVICE
            EPSHandle=NOVUSB('LastDevEPS.mat', ARG);
         end;
      end;
      
      function close
         global EPSHandle
         EPSHandle.close
      end;
      
      function [res ok] = read(addr)
         
         global EPSHandle
         
         [res ok] = EPSHandle.read(addr);
         
         if ok==0, % su success in reading. Try to connect again
            EPS1000.init;
            [res ok] = EPSHandle.read(addr);
         end;
         
      end;
      
      function [res] = setbitmode(mask, mode)
         
         global EPSHandle
         
         [res] = EPSHandle.setbitmode(mask, mode);
         
      end;
	  
	  function [dout1 ok] = readburst(rdaddr, addrstart, addrstop, wraddr)
	  
         global EPSHandle
         [dout1 ok] = EPSHandle.readburst(rdaddr, addrstart, addrstop, wraddr);
         if ok==0, % su success in reading. Try to connect again
            EPS1000.init;
            [dout1 ok] = EPSHandle.readburst(rdaddr, addrstart, addrstop, wraddr);
         end;
		end;

      function ok = write(addr, data)
      
         global EPSHandle
         ok = EPSHandle.write(addr, data);
         if ok==0, % su success in writing. Try to connect again
            EPS1000.init;
            [res ok] = EPSHandle.write(addr, data);
         end;
         
         % pause to force gap between write commands
         pausetime = 0.0010; % pause 1 ms at FT232R
         mstic=tic;
         mspause=0;
         while mspause<pausetime,
            mspause=toc(mstic);
         end;
         
         
      end;
      
    
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      % General instrument data %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      function r = getfirmware % as string
         r = sprintf('%4X', EPS1000.read(84));
      end
      
      
      function r = getserialnumber
         r = EPS1000.read(91);
      end;
      
      function r = getmoduletype % as string
         str=[];
         for ii=0:15,
            dummy = EPS1000.read(96+ii);
            str=[ str char(bitshift(dummy, -8)) ];
            str=[ str mod(dummy, 2^8) ];
         end;
         r = str;
      end
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Device specific functions %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      function r = gettemperature
         r = EPS1000.read(181)/16;
      end;
      
      function r = getnrofwlenbands
         r = EPS1000.read(190);
      end;
      
      
      
      function r = getoptfrequency(freq) % in THz
         minfreq=EPS1000.read(193)/2^7
         maxfreq=EPS1000.read(192)/2^7
         steps=EPS1000.read(194)
         stepsize=(maxfreq-minfreq)/steps;
         index=EPS1000.read(25);
         freq = minfreq+stepsize*index;
         r=freq;
         
      end;
      
      function r = setoptfrequency(freq) % in THz
         minfreq=EPS1000.read(193)/2^7;
         maxfreq=EPS1000.read(192)/2^7;
         steps=EPS1000.read(194);
         stepsize=(maxfreq-minfreq)/steps;
         index=round((freq-minfreq)/stepsize);
         if (index>=0) & (index<=steps)
            EPS1000.write(25, index);
         else
            disp('Freq. exceeds range!');
         end;
         
      end;
      
      function setposition(wp, position) % as integer 0..2^16-1
         %0: HWP
         %1..6: QWP0..QWP5
         EPS1000.write(40+wp, round(position));
      end;
      
      function r=getposition(wp) % as integer 0..2^16-1
         %0: HWP
         %1..6: QWP0..QWP5
         r = EPS1000.read(40+wp);
      end;
      
      function setspeed(wp, speed) % nominal speed in rad/s
         %0: HWP
         %1..6: QWP0..QWP5
         EPS1000.write(10+2*wp, 0); % clear MSB register first to avoid overflow
         
         if wp==0, % HWP
            speed = min(20000000, speed); % 20 Mrad/s max
            speed_fpga = round(speed/10);
         else
            speed = min(2000000, speed); % 2 Mrad/s max
            speed_fpga = round(speed*100);
         end;
         speed_fpga
         EPS1000.write(9+2*wp, mod(speed_fpga, 2^16)); % LSB
         EPS1000.write(10+2*wp, bitshift(speed_fpga, -16)); % MSB

      end;
      
      function r=getspeed(wp) % nominal speed in rad/s
         %0: HWP
         %1..6: QWP0..QWP5
         r = EPS1000.read(10+2*wp)*2^16 + EPS1000.read(9+2*wp);
         if wp==0, % HWP
            r = r*10;
         else
            r = r/100;
         end;
      end;
      
      function setrotation(wp, backw_en)
         %0: HWP
         %1..6: QWP0..QWP5
         %0, 2: STOP
         %1, FORWARD
         %3, BACKWARD
         EPS1000.write(wp, backw_en);
      end;

      function setrotation_all(backw_en)
         %backw_en is a vector of 7 values for 7 waveplates
         val=0;
         for ii=1:length(backw_en)
            val = val + backw_en(ii)*2^(2*(ii-1));
         end;
         EPS1000.write(7, val);
      end;
      
      function setpeakeddistr(speed) % in rad/s, requires firmware >=1.2.0.7
         israyleigh=0;
         target_fpga = round(speed/10);
         EPS1000.write(24, 0); % clear MSB register to avoid overflow
         EPS1000.write(23, mod(target_fpga, 2^16));
         EPS1000.write(24, 1*2^15 + israyleigh*2^14 + bitshift(target_fpga, -16));
      end;
      
      function setrayleighdistr(speed) % in rad/s, requires firmware >=1.2.0.7
         israyleigh=1;
         target_fpga = round(speed/10);
         EPS1000.write(24, 0); % clear MSB register to avoid overflow
         EPS1000.write(23, mod(target_fpga, 2^16));
         EPS1000.write(24, 1*2^15 + israyleigh*2^14 + bitshift(target_fpga, -16));
      end;
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%
      % Synchronous/Triggered %
      %%%%%%%%%%%%%%%%%%%%%%%%%
      
      function r=getexecutiontime_40ns % in steps of 40 ns
      
         r= EPS1000.read(222) + EPS1000.read(223) * 2^16;

      end
      
      function setexecutiontime_40ns(Execution_Time_Int) % in steps of 40 ns
      
         %Execution_Time_Int=25000; % 25000 * 40 ns = 1 ms
         %Execution_Time_Int=2500; % 2500 * 40 ns = 100 us
         %Execution_Time_Int=500; % 500 * 40 ns = 20 us
         EPS1000.write(222, mod(Execution_Time_Int, 2^16)); % lower 16 Bits
         EPS1000.write(223, bitshift(Execution_Time_Int, -16)); % upper 16 Bits

      end

      
      
      %%%%%%%%%%%%%%%%%%%%%
      % Special functions %
      %%%%%%%%%%%%%%%%%%%%%
      
      function setfanhysteresis(hyst)
         EPS1000.write(180, round(hyst));
      end;
      
      
      
      % writes voltage table to EPS
      function WriteTable(V, dwell_time_ns, nvolt)

         % determine the number of table entries (rows)
         npos=size(V,1);
         
         if length(dwell_time_ns)==1,
            dwell_time_ns=ones(1, npos)*dwell_time_ns;
         end;
            
            
         if nvolt==0, % write all voltages of mvolt
            xvolt = 1:size(V,2);
         else
            xvolt = nvolt; % write only specified row of mvolt
         end;

         for ii=1:npos
            EPS1000.write(219, ii-1); % select row
            % dwell time in 40ns-ticks:
            dwell_time_ticks=max(4, round(dwell_time_ns(ii)/40)-1);
            EPS1000.write(250, mod(dwell_time_ticks, 2^16)); % lower 16 Bits
            EPS1000.write(251, bitshift(dwell_time_ticks, -16)); % upper 16 Bits
            trigger = 1;
            % table data:
            for iii=xvolt
               EPS1000.write(252+iii-1, V(ii,iii)); % store current electrode voltages in registers
               trigger = bitor(trigger, 2^(iii-1));
            end;
            % Table write trigger
            EPS1000.write(221, trigger);
         end;
         
      end;
      
      
      % control
      function setcontrolenable(en)
         if en==1,
            EPS1000.write(340, 3); % this also sets tracking mode
         end;
         if en==0,
            dummy = EPS1000.read(340);
            EPS1000.write(340, dummy - mod(dummy,2)); % disable control but leave EPS in tracking mode
         end;
      end;
      
      function settrackingmode(en) % this overwrites the control enable bit
         en = min(en, 1);
         EPS1000.write(340, en*2);
      end;
      
      function showcontrolparameters()
         gain       = EPS1000.read(341);
         maximize   = EPS1000.read(342);
         ATE        = EPS1000.read(343);
         delay      = EPS1000.read(344);
         ditherampl = EPS1000.read(345);
         pullback   = EPS1000.read(346);
         mode = EPS1000.getcontrolmode();
         disp(sprintf('Detector Mode:        %d', mode.detector));
         disp(sprintf('Gradient Type:        %d', mode.gradtype));
         disp(sprintf('Reference Path:       %d', mode.reference));
         disp(sprintf('Sensitive Mode:       %d', mode.sensitive));
         disp(sprintf('APD Control:          %d', mode.apdcontrol));
         disp(sprintf('Control Gain:         %d', gain));
         disp(sprintf('Maximization:         %d', maximize));
         disp(sprintf('Averaging Time Exp.:  %d', ATE));
         disp(sprintf('Error Signal Delay:   %d', delay));
         disp(sprintf('Dither Amplitude:     %d', ditherampl));
         disp(sprintf('Pull Back Strength:   %d', pullback));
       end;
         
         
      
      function setcontrolgain(gain)
         EPS1000.write(341, gain);
      end;
      
      function setmaximize(maximize)
         maximize = min(maximize, 1);
         EPS1000.write(342, maximize);
      end;
      
      function setATE(ATE)
         EPS1000.write(343, ATE);
      end
      
      function setdelay(delay)
         EPS1000.write(344, delay);
      end
      
      function setrelmodstep(step)
         EPS1000.write(345, step);
      end
      
      function setrelpullstep(step)
         EPS1000.write(346, step);
      end
      
      function setcontrolmode(mode)
         EPS1000.write(347, mode.apdcontrol*2^6 + mode.sensitive*2^4 + mode.reference*2^3 + mode.gradtype*2^2 + mode.detector);
      end;
      
      function mode=getcontrolmode()
         dummy=EPS1000.read(347);
         mode.detector   = mod(dummy, 2^2);
         mode.gradtype   = mod(bitshift(dummy, -2), 2^1);
         mode.reference  = mod(bitshift(dummy, -3), 2^1);
         mode.sensitive  = mod(bitshift(dummy, -4), 2^2);
         mode.apdcontrol = mod(bitshift(dummy, -6), 2^1);
      end;
         
         
      
      
      
      
      % histogram during control
      function sethistogramoffset(offset)
         EPS1000.write(337, offset);
      end;
      
      function histogramstart()
         EPS1000.write(337, 1);
      end;
      
      function histogramstop()
         EPS1000.write(337, 2);
      end;
      
      function hist = histogramload()
         hist_help1 = EPS1000.readburst(339, 2^15+0, 2^15+1023, 339);
         hist_help2 = EPS1000.readburst(339, 2^14+0, 2^14+1023, 339);
         hist_help3 = EPS1000.readburst(339,      0,      1023, 339);
         for i=1:1024,
            hist(i) = hist_help1(i)*2^32 + hist_help2(i)*2^16 + hist_help3(i);
         end;
      end;
      
      
      

   end;

end